@@ -1,7 +1,3 @@
-/*
- * $Id: boundary.c,v 1.50 2017/03/09 07:10:52 sfeam Exp $
- */
-
 /* GNUPLOT - boundary.c */
 
 /*[
@@ -231,9 +227,12 @@ boundary(struct curve_points *plots, int count)
 	if (vertical_timelabel) {
 	    timelabel_textheight = timelabel_textwidth * t->v_char;
 	    timelabel_textwidth = (timelin + 1.5) * t->h_char;
+	    timelabel.place.y = 0;
 	} else {
 	    timelabel_textheight = timelin * t->v_char;
 	    timelabel_textwidth = timelabel_textwidth * t->h_char;
+	    /* save textheight for use in do_key_bounds() */
+	    timelabel.place.y = timelabel_textheight;
 	}
     }
 
@@ -244,8 +243,12 @@ boundary(struct curve_points *plots, int count)
 	    if (!axis_array[FIRST_Y_AXIS].ticmode)
 		ylabel_textwidth += 0.5 * t->v_char;
 	} else {
-	    /* How on earth did this used to work when it was always set to 0? */
-	    ylabel_textwidth = estimate_strlen(axis_array[FIRST_Y_AXIS].label.text) * t->h_char;
+	    /* Trying to estimate this length caused more problems than it solved.
+	     * For one thing it comes out wrong for text passed to TeX terminals.
+	     * Assume the common case is roughly 3 character widths and let the
+	     * user adjust lmargin and offset for longer non-rotated ylabels.
+	     */
+	    ylabel_textwidth = 3 * t->h_char;
 	}
     }
 
@@ -256,9 +259,8 @@ boundary(struct curve_points *plots, int count)
 	    if (!axis_array[SECOND_Y_AXIS].ticmode)
 		y2label_textwidth += 0.5 * t->v_char;
 	} else {
-	    /* should be (estimate_strlen()*t->h_char) but that's not what 5.0 did */
-	    /* this is an ugly ad hoc approximation of what it did before */
-	    y2label_textwidth = estimate_strlen(axis_array[SECOND_Y_AXIS].label.text) * t->h_char;
+	    /* See above. Estimating true text length causes problems */
+	    y2label_textwidth = 3 * t->h_char;
 	}
     }
 
@@ -628,9 +630,10 @@ boundary(struct curve_points *plots, int count)
 	} else
 	    current_aspect_ratio = aspect_ratio;
 
-	/* Set aspect ratio if valid and sensible */
-	/* EAM Mar 2008 - fixed borders take precedence over centering */
-	if (current_aspect_ratio >= 0.01 && current_aspect_ratio <= 100.0) {
+	if (current_aspect_ratio < 0.001 || current_aspect_ratio > 1000.0)
+	    int_warn(NO_CARET, "extreme aspect ratio");
+	if (TRUE) {
+	    /* Apply despite the warning */
 	    double current = ((double) (plot_bounds.ytop - plot_bounds.ybot))
 			   / (plot_bounds.xright - plot_bounds.xleft);
 	    double required = (current_aspect_ratio * t->v_tic) / t->h_tic;
@@ -747,9 +750,10 @@ boundary(struct curve_points *plots, int count)
 	+ ((float)xlablin+0.2) * t->v_char;
     xlabel_y -= ttic_textheight;
     ylabel_x = plot_bounds.xleft - ytic_width - ytic_textwidth;
-    ylabel_x -= ylabel_textwidth;
+    ylabel_x -= ylabel_textwidth/2;
 
     y2label_x = plot_bounds.xright + y2tic_width + y2tic_textwidth;
+    y2label_x += y2label_textwidth/2;
 
     /* Nov 2016  - simplify placement of timestamp
      * Stamp the same place on the page regardless of plot margins
@@ -852,6 +856,8 @@ do_key_bounds(legend_key *key)
 	} else if (key->margin == GPKEY_BMARGIN) {
 	    /* align bottom first since bmargin may be manual */
 	    key->bounds.ybot = yoffset * t->ymax + t->v_tic;
+	    if (timelabel.rotate == 0 && timelabel_bottom && timelabel.place.y > 0)
+		key->bounds.ybot += (int)(timelabel.place.y);
 	    key->bounds.ytop = key->bounds.ybot + key_height;
 	} else {
 	    if (key->vpos == JUST_TOP) {
@@ -1248,7 +1254,7 @@ do_key_sample(
     } else if (this_plot->lp_properties.l_type == LT_NODRAW) {
 	;
 
-    } else if ((this_plot->plot_style & PLOT_STYLE_HAS_ERRORBAR) &&  this_plot->plot_type == DATA) {
+    } else if ((this_plot->plot_style & PLOT_STYLE_HAS_ERRORBAR) && this_plot->plot_type != FUNC) {
 	/* errors for data plots only */
 	if ((bar_lp.flags & LP_ERRORBAR_SET) != 0)
 	    term_apply_lp_properties(&bar_lp);
@@ -1261,7 +1267,7 @@ do_key_sample(
 	draw_clip_line(xl + key_sample_left, yl, xl + key_sample_right, yl);
     }
 
-    if ((this_plot->plot_type == DATA)
+    if ((this_plot->plot_type == DATA || this_plot->plot_type == KEYENTRY)
 	&& (this_plot->plot_style & PLOT_STYLE_HAS_ERRORBAR)
 	&& (this_plot->plot_style != CANDLESTICKS)
 	&& (bar_size > 0.0)) {
@@ -1419,21 +1425,14 @@ draw_titles()
 	unsigned int x = ylabel_x;
 	unsigned int y = (plot_bounds.ytop + plot_bounds.ybot) / 2;
 	x += t->h_char;
-	axis_array[FIRST_Y_AXIS].label.pos = CENTRE;
 	write_label(x, y, &(axis_array[FIRST_Y_AXIS].label));
 	reset_textcolor(&(axis_array[FIRST_Y_AXIS].label.textcolor));
     }
 
     /* Y2LABEL */
     if (axis_array[SECOND_Y_AXIS].label.text) {
-	unsigned int x = y2label_x;
-	unsigned int y = (plot_bounds.ytop + plot_bounds.ybot) / 2;
-	if (axis_array[SECOND_Y_AXIS].label.rotate) {
-	    x += 2 * t->h_char;
-	    axis_array[SECOND_Y_AXIS].label.pos = CENTRE;
-	} else {
-	    axis_array[SECOND_Y_AXIS].label.pos = LEFT;
-	}
+	int x = y2label_x;
+	int y = (plot_bounds.ytop + plot_bounds.ybot) / 2;
 	write_label(x, y, &(axis_array[SECOND_Y_AXIS].label));
 	reset_textcolor(&(axis_array[SECOND_Y_AXIS].label.textcolor));
     }
