@@ -41,6 +41,7 @@
 #include "datablock.h"	/* for datablock_size() */
 #include "variable.h"	/* for locale handling */
 #include "parse.h"	/* for string_result_only */
+#include "datafile.h"	/* for evaluate_inside_using */
 
 #include <math.h>
 
@@ -79,6 +80,7 @@ void
 eval_reset_after_error()
 {
     recursion_depth = 0;
+    undefined = FALSE;
 }
 
 void
@@ -1483,7 +1485,8 @@ f_sprintf(union argument *arg)
 
     /* If the user has set an explicit LC_NUMERIC locale, apply it */
     /* to sprintf calls during expression evaluation.              */
-    set_numeric_locale();
+    if (!evaluate_inside_using)
+	set_numeric_locale();
 
     /* Each time we start this loop we are pointing to a % character */
     while (remaining-->0 && next_start[0] && next_start[1]) {
@@ -1579,7 +1582,8 @@ f_sprintf(union argument *arg)
 	free(args);
 
     /* Return to C locale for internal use */
-    reset_numeric_locale();
+    if (!evaluate_inside_using)
+	reset_numeric_locale();
 
 }
 
