@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.20 2014/06/15 10:33:33 markisch Exp $"); }
-#endif
-
 /* GNUPLOT - plot.c */
 
 /*[
@@ -72,12 +68,7 @@ static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.20 2014/06/15 10:33:3
 # include "gpexecute.h"
 #endif
 
-/* Used nowhere else */
-#ifdef HAVE_SYS_UTSNAME_H
-# include <sys/utsname.h>
-#endif
-
-#if defined(MSDOS) || defined(__EMX__)
+#if defined(MSDOS) || defined(__EMX__) || (defined(WGP_CONSOLE) && defined(MSVC))
 # include <io.h>
 #endif
 
@@ -93,14 +84,12 @@ extern int vms_ktid;
 extern smg$create_key_table();
 #endif /* VMS */
 
-#ifdef _Windows
+#ifdef _WIN32
 # include <windows.h>
-# ifndef SIGINT
-#  define SIGINT 2		/* for MSC */
-# endif
 # include "win/winmain.h"
 # include "win/wcommon.h"
-#endif /* _Windows */
+# include <io.h>           // for isatty
+#endif /* _WIN32 */
 
 /* GNU readline
  * Only required by two files directly,
@@ -136,9 +125,10 @@ static void wrapper_for_write_history __PROTO((void));
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
 TBOOLEAN noinputfiles = TRUE;	/* FALSE if there are script files */
-TBOOLEAN reading_from_dash=FALSE; /* True if processing "-" as an input file */
-TBOOLEAN skip_gnuplotrc = FALSE;/* skip system gnuplotrc and ~/.gnuplot */
-TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
+TBOOLEAN reading_from_dash=FALSE;	/* True if processing "-" as an input file */
+TBOOLEAN skip_gnuplotrc = FALSE;	/* skip system gnuplotrc and ~/.gnuplot */
+TBOOLEAN persist_cl = FALSE; 		/* --persist command line option */
+TBOOLEAN slow_font_startup = FALSE;	/* --slow command line option */
 
 /* user home directory */
 static const char *user_homedir = NULL;
@@ -159,13 +149,7 @@ char HelpFile[MAXPATH];
 #endif /*   - DJL */
 
 /* a longjmp buffer to get back to the command line */
-/* FIXME HBB 20001103: should probably just use GPFAR, rather than
- * check for _Windows */
-#ifdef _Windows
-static JMP_BUF far command_line_env;
-#else
 static JMP_BUF command_line_env;
-#endif
 
 static void load_rcfile __PROTO((int where));
 static RETSIGTYPE inter __PROTO((int anint));
@@ -173,6 +157,11 @@ static void init_memory __PROTO((void));
 
 static int exit_status = EXIT_SUCCESS;
 
+/* Flag for asynchronous handling of Ctrl-C. Used by fit.c and Windows */
+TBOOLEAN ctrlc_flag = FALSE;
+/* Flag for (asynchronous) term signal on Windows. */
+TBOOLEAN terminate_flag = FALSE;
+
 #ifdef OS2
 # include <process.h>
 static ULONG RexxInterface(PRXSTRING, PUSHORT, PRXSTRING);
@@ -195,11 +184,22 @@ inter(int anint)
 	 */
     } else
 #endif
+#if defined(WGP_CONSOLE)
+	/* The Windows console Ctrl-C handler runs in another thread. So a
+	   longjmp() would result in crash. Instead, we handle these
+	   events asynchronously.
+	*/
+	ctrlc_flag = TRUE;
+	/* Interrupt ConsoleGetch. */
+	SendMessage(graphwin->hWndGraph, WM_NULL, 0, 0);
+	SendMessage(GetConsoleWindow(), WM_CHAR, 0x20, 0);
+#else
     {
     term_reset();
     (void) putc('\n', stderr);
     bail_to_command_line();	/* return to prompt */
     }
+#endif
 }
 
 #ifdef LINUXVGA
@@ -255,13 +255,14 @@ take_privilege()
 void
 bail_to_command_line()
 {
-#ifdef _Windows
-    call_kill_pending_Pause_dialog();
+#ifdef _WIN32
+    kill_pending_Pause_dialog();
+    ctrlc_flag = FALSE;
 #endif
     LONGJMP(command_line_env, TRUE);
 }
 
-#if defined(_Windows)
+#if defined(_WIN32)
 int
 gnu_main(int argc, char **argv)
 #else
@@ -278,10 +279,13 @@ main(int argc, char **argv)
 /* make sure that we really have revoked root access, this might happen if
    gnuplot is compiled without vga support but is installed suid by mistake */
 #ifdef __linux__
-    setuid(getuid());
+    if (setuid(getuid()) != 0) {
+	fprintf(stderr,"gnuplot: refusing to run at elevated privilege\n");
+	exit(EXIT_FAILURE);
+    }
 #endif
 
-#if defined(MSDOS) && !defined(_Windows) && !defined(__GNUC__)
+#if defined(MSDOS) && !defined(_WIN32) && !defined(__GNUC__)
     PC_setup();
 #endif /* MSDOS !Windows */
 
@@ -292,15 +296,18 @@ main(int argc, char **argv)
 #endif
 
 #if defined(OS2)
-    int rc;
+    {
+	int rc;
 #ifdef OS2_IPC
-    char semInputReadyName[40];
-    sprintf( semInputReadyName, "\\SEM32\\GP%i_Input_Ready", getpid() );
-    rc = DosCreateEventSem(semInputReadyName,&semInputReady,0,0);
-    if (rc != 0)
-      fputs("DosCreateEventSem error\n",stderr);
+	char semInputReadyName[40];
+
+	sprintf(semInputReadyName, "\\SEM32\\GP%i_Input_Ready", getpid());
+	rc = DosCreateEventSem(semInputReadyName, &semInputReady, 0, 0);
+	if (rc != 0)
+	    fputs("DosCreateEventSem error\n", stderr);
 #endif
-    rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
+	rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
+    }
 #endif
 
 /* malloc large blocks, otherwise problems with fragmented mem */
@@ -324,7 +331,10 @@ main(int argc, char **argv)
     unsigned int status[2] = { 1, 0 };
 #endif
 
-#if defined(HAVE_LIBEDITLINE)
+#if (defined(PIPE_IPC) || defined(_WIN32)) && (defined(HAVE_LIBREADLINE) || (defined(HAVE_LIBEDITLINE) && defined(X11)))
+    /* Editline needs this to be set before the very first call to readline(). */
+    /* Support for rl_getc_function is broken for utf-8 in editline. Since it is only
+       really required for X11, disable this section when building without X11. */
     rl_getc_function = getc_wrapper;
 #endif
 
@@ -355,14 +365,16 @@ main(int argc, char **argv)
 	    return 0;
 
 	} else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
-	    printf( "Usage: gnuplot [OPTION]... [FILE]\n"
+	    printf( "Usage: gnuplot [OPTION] ... [FILE]\n"
 #ifdef X11
 		    "for X11 options see 'help X11->command-line-options'\n"
 #endif
 		    "  -V, --version\n"
 		    "  -h, --help\n"
 		    "  -p  --persist\n"
+		    "  -s  --slow\n"
 		    "  -d  --default-settings\n"
+		    "  -c  scriptfile ARG1 ARG2 ... \n"
 		    "  -e  \"command1; command2; ...\"\n"
 		    "gnuplot %s patchlevel %s\n",
 		    gnuplot_version, gnuplot_patchlevel);
@@ -379,11 +391,13 @@ main(int argc, char **argv)
 	    return 0;
 
 	} else if (!strncmp(argv[i], "-persist", 2) || !strcmp(argv[i], "--persist")
-#ifdef _Windows
+#ifdef _WIN32
 		|| !stricmp(argv[i], "-noend") || !stricmp(argv[i], "/noend")
 #endif
 		) {
 	    persist_cl = TRUE;
+	} else if (!strncmp(argv[i], "-slow", 2) || !strcmp(argv[i], "--slow")) {
+	    slow_font_startup = TRUE;
 	} else if (!strncmp(argv[i], "-d", 2) || !strcmp(argv[i], "--default-settings")) {
 	    /* Skip local customization read from ~/.gnuplot */
 	    skip_gnuplotrc = TRUE;
@@ -416,29 +430,25 @@ main(int argc, char **argv)
      * EAM - Jan 2013 YES.
      */
     setvbuf(stdin, (char *) NULL, _IONBF, 0);
-
 #endif
 
     gpoutfile = stdout;
 
     /* Initialize pre-loaded user variables */
-    (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
-    udv_NaN = add_udv_by_name("NaN");
-    (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
-    udv_NaN->udv_undef = FALSE;
+    /* "pi" is hard-wired as the first variable */
+    (void) add_udv_by_name("GNUTERM");
+    (void) add_udv_by_name("NaN");
+    init_constants();
+    udv_user_head = &(udv_NaN->next_udv);
 
     init_memory();
 
     interactive = FALSE;
-    init_terminal();		/* can set term type if it likes */
-    push_terminal(0);		/* remember the default terminal */
 
-    /* reset the terminal when exiting */
-    /* this is done through gp_atexit so that other terminal functions
-     * can be registered to be executed before the terminal is reset. */
-    GP_ATEXIT(term_reset);
+    /* April 2017:  We used to call init_terminal() here, but now   */
+    /* We defer initialization until error handling has been set up. */
 
-# if defined(_Windows) && ! defined(WGP_CONSOLE)
+# if defined(_WIN32) && !defined(WGP_CONSOLE)
     interactive = TRUE;
 # else
     interactive = isatty(fileno(stdin));
@@ -451,11 +461,11 @@ main(int argc, char **argv)
      * command line arguments are file names or an explicit in-line "-e command".
      */
     for (i = 1; i < argc; i++) {
-# ifdef _Windows
+# ifdef _WIN32
 	if (!stricmp(argv[i], "/noend"))
 	    continue;
 # endif
-	if ((argv[i][0] != '-') || (argv[i][1] == 'e')) {
+	if ((argv[i][0] != '-') || (argv[i][1] == 'e') || (argv[i][1] == 'c') ) {
 	    interactive = FALSE;
 	    break;
 	}
@@ -463,69 +473,11 @@ main(int argc, char **argv)
 
     /* Need this before show_version is called for the first time */
 
-#ifdef HAVE_SYS_UTSNAME_H
-    {
-	struct utsname uts;
-
-	/* something is fundamentally wrong if this fails ... */
-	if (uname(&uts) > -1) {
-# ifdef _AIX
-	    strcpy(os_name, uts.sysname);
-	    sprintf(os_name, "%s.%s", uts.version, uts.release);
-# elif defined(SCO)
-	    strcpy(os_name, "SCO");
-	    strcpy(os_rel, uts.release);
-# elif defined(DJGPP)
-	    if (!strncmp(uts.sysname, "??Un", 4)) /* don't print ??Unknow" */
-		strcpy(os_name, "Unknown");
-	    else {
-		strcpy(os_name, uts.sysname);
-		strcpy(os_rel, uts.release);
-	    }
-# else
-	    strcpy(os_name, uts.sysname);
-	    strcpy(os_rel, uts.machine);
-# ifdef OS2
-	    if (!strchr(os_rel,'.'))
-		/* write either "2.40" or "4.0", or empty -- don't print "OS/2 1" */
-		strcpy(os_rel, "");
-# endif
-
-# endif
-	}
-    }
-#else /* ! HAVE_SYS_UTSNAME_H */
-
-    strcpy(os_name, OS);
-    strcpy(os_rel, "");
-
-#endif /* HAVE_SYS_UTSNAME_H */
-
     if (interactive)
 	show_version(stderr);
     else
 	show_version(NULL); /* Only load GPVAL_COMPILE_OPTIONS */
 
-#ifdef WGP_CONSOLE
-#ifdef CONSOLE_SWITCH_CP
-    if (cp_changed && interactive) {
-	fprintf(stderr,
-	    "\ngnuplot changed the codepage of this console from %i to %i to\n" \
-	    "match the graph window. Some characters might only display correctly\n" \
-	    "if you change the font to a non-raster type.\n",
-	    cp_input, GetConsoleCP());
-    }
-#else
-    if ((GetConsoleCP() != GetACP()) && interactive) {
-	fprintf(stderr,
-	    "\nWarning: The codepage of the graph window (%i) and that of the\n" \
-	    "console (%i) differ. Use `set encoding` or `!chcp` if extended\n" \
-	    "characters don't display correctly.\n",
-	    GetACP(), GetConsoleCP());
-    }
-#endif
-#endif
-
     update_gpval_variables(3);  /* update GPVAL_ variables available to user */
 
 #ifdef VMS
@@ -539,56 +491,53 @@ main(int argc, char **argv)
     if (!SETJMP(command_line_env, 1)) {
 	/* first time */
 	interrupt_setup();
-	/* should move this stuff another initialisation routine,
-	 * something like init_set() maybe */
 	get_user_env();
 	init_loadpath();
 	init_locale();
-	/* HBB: make sure all variables start in the same mode 'reset'
-	 * would set them to. Since the axis variables aren't in
-	 * initialized arrays any more, this is now necessary... */
-	reset_command();
-	init_color();		/* Initialization of color  */
-	load_rcfile(0);		/* System-wide gnuplotrc if configured */
-	load_rcfile(1);		/* ./.gnuplot if configured */
+
+	memset(&sm_palette, 0, sizeof(sm_palette));
 	init_fit();		/* Initialization of fitting module */
+#ifdef READLINE
+	/* When using the built-in readline, we set the initial
+	   encoding according to the locale as this is required
+	   to properly handle keyboard input. */
+	init_encoding();
+#endif
+	/* April 2017: Now that error handling is in place, it is safe parse
+	 * GNUTERM during terminal initialization.
+	 * atexit processing is done in reverse order. We want
+	 * the generic terminal shutdown in term_reset to be executed before
+	 * any terminal specific cleanup requested by individual terminals.
+	 */
+	init_terminal();
+	push_terminal(0);	/* remember the initial terminal */
+	gp_atexit(term_reset);
 
-	/* After this point we allow pipes and system commands */
-	successful_initialization = TRUE;
-
-	load_rcfile(2);		/* ~/.gnuplot */
+	/* Execute commands in ~/.gnuplot */
+	init_session();
 
 	if (interactive && term != 0) {		/* not unknown */
 #ifdef GNUPLOT_HISTORY
-	    FPRINTF((stderr, "Before read_history\n"));
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && !defined(_WIN32)
 	    expanded_history_filename = tilde_expand(GNUPLOT_HISTORY_FILE);
 #else
 	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
 	    gp_expand_tilde(&expanded_history_filename);
 #endif
-	    FPRINTF((stderr, "expanded_history_filename = %s\n", expanded_history_filename));
 	    read_history(expanded_history_filename);
-	    {
-		/* BEGIN: Go local to get environment variable */
-		const char *temp_env = getenv ("GNUPLOT_HISTORY_SIZE");
-		if (temp_env)
-		    gnuplot_history_size = strtol (temp_env, (char **) NULL, 10);
-	    } /* END: Go local to get environment variable */
 
 	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
 	     * 'on_exit()'. In the worst case, there is no history of your
 	     * currrent session and you have to type all again in your next
 	     * session.
-	     * This is the default behaviour (traditional reasons), too.
-	     * In case you don't have one of these functions, or you don't
-	     * want to use them, 'write_history()' is called directly.
 	     */
-	    GP_ATEXIT(wrapper_for_write_history);
+	    gp_atexit(wrapper_for_write_history);
 #endif /* GNUPLOT_HISTORY */
 
-	    fprintf(stderr, "\nTerminal type set to '%s'\n", term->name);
+#if defined(READLINE) && defined(WGP_CONSOLE)
+	    fprintf(stderr, "Encoding set to '%s'.\n", encoding_names[encoding]);
+#endif
 	}			/* if (interactive && term != 0) */
     } else {
 	/* come back here from int_error() */
@@ -600,8 +549,7 @@ main(int argc, char **argv)
 	if (interactive == FALSE)
 	    exit_status = EXIT_FAILURE;
 #ifdef HAVE_READLINE_RESET
-	else
-	{
+	else {
 	    /* reset properly readline after a SIGINT+longjmp */
 	    rl_reset_after_signal ();
 	}
@@ -640,7 +588,7 @@ main(int argc, char **argv)
 
 	if (!interactive && !noinputfiles) {
 	    term_reset();
-	    exit(EXIT_FAILURE);	/* exit on non-interactive error */
+	    gp_exit(EXIT_FAILURE);	/* exit on non-interactive error */
 	}
     }
 
@@ -649,13 +597,13 @@ main(int argc, char **argv)
 	    ++argv;
 	    c_token = 0;
 	    if (!strncmp(*argv, "-persist", 2) || !strcmp(*argv, "--persist")
-#ifdef _Windows
+#ifdef _WIN32
 		|| !stricmp(*argv, "-noend") || !stricmp(*argv, "/noend")
 #endif
 	    ) {
 		FPRINTF((stderr,"'persist' command line option recognized\n"));
 	    } else if (strcmp(*argv, "-") == 0) {
-#if defined(_Windows) && !defined(WGP_CONSOLE)
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
 		TextShow(&textwin);
 		interactive = TRUE;
 #else
@@ -678,15 +626,39 @@ RECOVER_FROM_ERROR_IN_DASH:
 		noinputfiles = FALSE;
 		do_string(*argv);
 
+	    } else if (!strncmp(*argv, "-slow", 2) || !strcmp(*argv, "--slow")) {
+		slow_font_startup = TRUE;
+
 	    } else if (!strncmp(*argv, "-d", 2) || !strcmp(*argv, "--default-settings")) {
 		/* Ignore this; it already had its effect */
 		FPRINTF((stderr, "ignoring -d\n"));
+
+	    } else if (strcmp(*argv, "-c") == 0) {
+		/* Pass command line arguments to the gnuplot script in the next
+		 * argument. This consumes the remainder of the command line
+		 */
+		interactive = FALSE;
+		noinputfiles = FALSE;
+		--argc; ++argv;
+		if (argc <= 0) {
+		    fprintf(stderr, "syntax:  gnuplot -c scriptname args\n");
+		    gp_exit(EXIT_FAILURE);
+		}
+		call_argc = GPMIN(9, argc - 1);
+		for (i=0; i<=call_argc; i++) {
+		    /* Need to stash argv[i] somewhere visible to load_file() */
+		    call_args[i] = gp_strdup(argv[i+1]);
+		}
+
+		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 5);
+		gp_exit(EXIT_SUCCESS);
+
 	    } else if (*argv[0] == '-') {
 		fprintf(stderr, "unrecognized option %s\n", *argv);
 	    } else {
 		interactive = FALSE;
 		noinputfiles = FALSE;
-		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), FALSE);
+		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 4);
 	    }
     }
 
@@ -695,7 +667,7 @@ RECOVER_FROM_ERROR_IN_DASH:
 	while (!com_line())
 	    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
 
-#ifdef _Windows
+#ifdef _WIN32
     /* On Windows, handle 'persist' by keeping the main input loop running (windows/wxt), */
     /* but only if there are any windows open. Note that qt handles this properly. */
     if (persist_cl) {
@@ -730,6 +702,10 @@ RECOVER_FROM_ERROR_IN_DASH:
 
     /* HBB 20040223: Not all compilers like exit() to end main() */
     /* exit(exit_status); */
+#if ! defined(_WIN32)
+    /* Windows does the cleanup later */
+    gp_exit_cleanup();
+#endif
     return exit_status;
 }
 
@@ -738,15 +714,7 @@ RECOVER_FROM_ERROR_IN_DASH:
 void
 interrupt_setup()
 {
-#if defined(WGP_CONSOLE)
-    /* FIXME. CTRC+C crashes console mode gnuplot for windows.
-       Failure of longjmp() is not easy to fix so that the signal
-       of SIGINT is just ignored at the moment.
-    */
-    (void) signal(SIGINT, SIG_IGN);
-#else
     (void) signal(SIGINT, (sigfunc) inter);
-#endif
 
 #ifdef SIGPIPE
     /* ignore pipe errors, this might happen with set output "|head" */
@@ -755,6 +723,46 @@ interrupt_setup()
 }
 
 
+/*
+ * Initialize 'constants' stored as variables (user could mangle these)
+ */
+void
+init_constants()
+{
+    (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
+    udv_NaN = get_udv_by_name("NaN");
+    (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
+}
+
+/*
+ * Initialize graphics context, color palette, local preferences.
+ * Called both at program startup and by "reset session".
+ */
+void
+init_session()
+{
+	/* Disable pipes and system commands during initialization */
+	successful_initialization = FALSE;
+
+	/* Undefine any previously-used variables */
+	del_udv_by_name("",TRUE);
+
+	/* Restore default colors before loadin local preferences */
+	set_colorsequence(1);
+
+	/* Make sure all variables start in the same state 'reset'
+	 * would set them to.
+	 */
+	reset_command();	/* FIXME: this does c_token++ */
+	load_rcfile(0);		/* System-wide gnuplotrc if configured */
+	load_rcfile(1);		/* ./.gnuplot if configured */
+
+	/* After this point we allow pipes and system commands */
+	successful_initialization = TRUE;
+
+	load_rcfile(2);		/* ~/.gnuplot */
+}
+
 /*
  * Read commands from an initialization file.
  * where = 0: look for gnuplotrc in system shared directory
@@ -772,18 +780,13 @@ load_rcfile(int where)
 
     if (where == 0) {
 #ifdef GNUPLOT_SHARE_DIR
-# if defined(_Windows)
-	/* retrieve path relative to gnuplot executable,
-	 * whose path is in szModuleName (winmain.c) */
-	rcfile = gp_alloc(strlen((char *)szPackageDir) + 1
-	       + strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
-	strcpy(rcfile, (char *)szPackageDir);
-	PATH_CONCAT(rcfile, GNUPLOT_SHARE_DIR);
+# if defined(_WIN32)
+	rcfile = RelativePathToGnuplot(GNUPLOT_SHARE_DIR "\\gnuplotrc");
 # else
 	rcfile = (char *) gp_alloc(strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
 	strcpy(rcfile, GNUPLOT_SHARE_DIR);
-# endif
 	PATH_CONCAT(rcfile, "gnuplotrc");
+# endif
 	plotrc = fopen(rcfile, "r");
 #endif
 
@@ -806,13 +809,14 @@ load_rcfile(int where)
 
     if (plotrc) {
 	char *rc = gp_strdup(rcfile ? rcfile : PLOTRC);
-	load_file(plotrc, rc, FALSE);
+	load_file(plotrc, rc, 3);
 	push_terminal(0); /* needed if terminal or its options were changed */
     }
 
     free(rcfile);
 }
 
+
 void
 get_user_env()
 {
@@ -837,7 +841,7 @@ get_user_env()
 	const char *env_shell;
 
 	if ((env_shell = getenv("SHELL")) == NULL)
-#if defined(MSDOS) || defined(_Windows) || defined(OS2)
+#if defined(MSDOS) || defined(_WIN32) || defined(OS2)
 	    if ((env_shell = getenv("COMSPEC")) == NULL)
 #endif
 		env_shell = SHELL;
@@ -876,8 +880,7 @@ init_memory()
 {
     extend_input_line();
     extend_token_table();
-    replot_line = gp_alloc(1, "string");
-    *replot_line = NUL;
+    replot_line = gp_strdup("");
 }
 
 
@@ -1058,7 +1061,7 @@ wrapper_for_write_history()
 	return;
     remove(expanded_history_filename);
     if (gnuplot_history_size < 0)
-    	write_history(expanded_history_filename);
+	write_history(expanded_history_filename);
     else
 	write_history_n(gnuplot_history_size, expanded_history_filename, "w");
 }
