@@ -1521,6 +1521,20 @@ eval_3dplots()
 
 		/*}}} */
 
+	    } else if (equals(c_token, "keyentry")) {
+		c_token++;
+		plot_num++;
+		if (*tp_3d_ptr)
+		    this_plot = *tp_3d_ptr;
+		else {		/* no memory malloc()'d there yet */
+		    /* Allocate enough isosamples and samples */
+		    this_plot = sp_alloc(0, 0, 0, 0);
+		    *tp_3d_ptr = this_plot;
+		}
+		this_plot->plot_type = KEYENTRY;
+		this_plot->plot_style = LABELPOINTS;
+		this_plot->token = end_token = c_token - 1;
+
 	    } else {		/* function to plot */
 
 		/*{{{  function */
@@ -1601,6 +1615,7 @@ eval_3dplots()
 		    if ((this_plot->plot_type == FUNC3D) &&
 			((this_plot->plot_style & PLOT_STYLE_HAS_ERRORBAR)
 			|| (this_plot->plot_style == LABELPOINTS && !draw_contour)
+			|| (this_plot->plot_style == VECTOR)
 			)) {
 			int_warn(c_token-1, "This style cannot be used to plot a surface defined by a function");
 			this_plot->plot_style = POINTSTYLE;
@@ -1850,6 +1865,14 @@ eval_3dplots()
 		    this_plot->lp_properties.p_size = 1;
 	    }
 
+	    /* FIXME: Leaving an explicit font in the label style for contour */
+	    /* labels causes a double-free segfault.  Clear it preemptively.  */
+	    if (this_plot->plot_style == LABELPOINTS
+	    &&  (draw_contour && !this_plot->opt_out_of_contours)) {
+		free(this_plot->labels->font);
+		this_plot->labels->font = NULL;
+	    }
+
 	    if (crnt_param == 0
 		&& this_plot->plot_style != PM3DSURFACE
 		/* don't increment the default line/point properties if
@@ -2158,7 +2181,9 @@ eval_3dplots()
 		dummy_func = &plot_func;
 		name_str = string_or_express(&at_ptr);
 
-		if (!name_str) {                /* func to plot */
+		if (equals(c_token, "keyentry")) {
+
+		} else if (!name_str) {                /* func to plot */
 		    /*{{{  evaluate function */
 		    struct iso_curve *this_iso = this_plot->iso_crvs;
 		    int num_sam_to_use, num_iso_to_use;
@@ -2310,6 +2335,10 @@ eval_3dplots()
 	    ||  this_plot->plot_style == RGBIMAGE
 	    ||  this_plot->plot_style == RGBA_IMAGE)
 		continue;
+	    if (this_plot->plot_type == NODATA)
+		continue;
+	    if (this_plot->plot_type == KEYENTRY)
+		continue;
 
 	    /* Allow individual surfaces to opt out of contouring */
 	    if (this_plot->opt_out_of_contours)
@@ -2504,8 +2533,6 @@ parametric_3dfixup(struct surface_points *start_plot, int *plot_num)
 static void load_contour_label_options (struct text_label *contour_label)
 {
     struct lp_style_type *lp = &(contour_label->lp_properties);
-    if (!contour_label->font)
-	contour_label->font = gp_strdup(clabel_font);
     lp->p_interval = clabel_interval;
     lp->flags |= LP_SHOW_POINTS;
     lp_parse(lp, LP_ADHOC, TRUE);
