@@ -385,9 +385,13 @@ void pm3d_depth_queue_flush(void)
 	double w = trans_mat[3][3];
 #endif
 	vertex out;
-	double z = 0; /* assignment keeps the compiler happy */
+	double z = 0;
+	double zbase = 0;
 	int i;
 
+	if (pm3d.base_sort)
+	    cliptorange(zbase, Z_AXIS.min, Z_AXIS.max);
+
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
 	    gpdPtr = qp->corners;
@@ -398,7 +402,7 @@ void pm3d_depth_queue_flush(void)
 	    for (i = 0; i < 4; i++, gpdPtr++) {
 
 		if (pm3d.base_sort)
-		    map3d_xyz(gpdPtr->x, gpdPtr->y, 0.0, &out);
+		    map3d_xyz(gpdPtr->x, gpdPtr->y, zbase, &out);
 		else
 		    map3d_xyz(gpdPtr->x, gpdPtr->y, gpdPtr->z, &out);
 
@@ -777,6 +781,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		if (isnan(avgC))
 		    continue;
 
+		/* Option to not drawn quadrangles with cb < cbmin */
+		if (pm3d.no_clipcb && (avgC < CB_AXIS.min))
+		    continue;
+
 		if (color_from_rgbvar) /* we were given an RGB color */
 			gray = avgC;
 		else /* transform z value to gray, i.e. to interval [0,1] */
@@ -1113,6 +1121,7 @@ pm3d_reset()
     pm3d.flush = PM3D_FLUSH_BEGIN;
     pm3d.ftriangles = 0;
     pm3d.clip = PM3D_CLIP_4IN;
+    pm3d.no_clipcb = FALSE;
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
     pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
