@@ -788,6 +788,7 @@ set origin %g,%g\n",
 	save_position(fp, &(lab.offset), 3, TRUE);			 \
 	fprintf(fp, " font \"%s\"", lab.font ? conv_text(lab.font) : "");\
 	save_textcolor(fp, &(lab.textcolor));				 \
+	if (lab.pos != CENTRE) save_justification(lab.pos,fp);		 \
 	if (lab.tag == ROTATE_IN_3D_LABEL_TAG)				 \
 	    fprintf(fp, " rotate parallel");				 \
 	else if (lab.rotate == TEXT_VERTICAL)				 \
@@ -907,6 +908,12 @@ set origin %g,%g\n",
     }
     fputs("\n", fp);
 
+#if (0)
+    fprintf(fp, "set pm3d %s %s\n",
+		pm3d.clip == PM3D_CLIP_1IN ? "clip1in" : "clip4in",
+		pm3d.no_clipcb ? "noclipcb" : "");
+#endif
+
     if (pm3d_shade.strength <= 0)
 	fputs("set pm3d nolighting\n",fp);
     else
@@ -931,7 +938,6 @@ set origin %g,%g\n",
 	case C_MODEL_RGB: fputs( "RGB ", fp ); break;
 	case C_MODEL_HSV: fputs( "HSV ", fp ); break;
 	case C_MODEL_CMY: fputs( "CMY ", fp ); break;
-	case C_MODEL_XYZ: fputs( "XYZ ", fp ); break;
       }
       fputs( "\nset palette ", fp );
       switch( sm_palette.colorMode ) {
@@ -1666,8 +1672,10 @@ save_linetype(FILE *fp, lp_style_type *lp, TBOOLEAN show_point)
 	    fprintf(fp, " pointsize default");
 	else
 	    fprintf(fp, " pointsize %.3f", lp->p_size);
-	fprintf(fp, " pointinterval %d", lp->p_interval);
-	fprintf(fp, " pointnumber %d", lp->p_number);
+	if (lp->p_interval != 0)
+	    fprintf(fp, " pointinterval %d", lp->p_interval);
+	if (lp->p_number != 0)
+	    fprintf(fp, " pointnumber %d", lp->p_number);
     }
 
 }
